@@ -1,7 +1,7 @@
 from bisect import bisect_left, bisect_right
 from typing import Any, Literal, Optional, Sequence, Tuple, TypeVar, Union
 
-from transformers.tokenization_utils_base import PreTrainedTokenizerBase
+from necessary import necessary, Necessary
 
 from smashed.base import BaseRecipe, SingleBaseMapper, TransformElementType
 from smashed.base.mappers import ChainableMapperMixIn
@@ -16,6 +16,10 @@ from smashed.mappers import (
 )
 from smashed.recipes.prompting import PromptingRecipe
 
+with necessary("transformers", soft=True):
+    from transformers.tokenization_utils_base import PreTrainedTokenizerBase
+
+
 __all__ = [
     "AddEvidencesLocationMapper",
     "ConcatenateContextMapper",
@@ -317,7 +321,7 @@ class _SquadPromptingRecipe(PromptingRecipe):
     def __init__(
         self,
         *args,
-        tokenizer: PreTrainedTokenizerBase,
+        tokenizer: 'PreTrainedTokenizerBase',
         context_field: str = "context",
         location_field: str = "locations",
         **kwargs,
@@ -331,13 +335,14 @@ class _SquadPromptingRecipe(PromptingRecipe):
 C = TypeVar("C", bound=ChainableMapperMixIn)
 
 
+@Necessary("transformers")
 class SquadPromptTrainRecipe(BaseRecipe):
     def unpacking(self, pipeline: C, **kwargs: Any) -> C:
         return pipeline >> UnpackingMapper(**kwargs)
 
     def __init__(
         self,
-        tokenizer: PreTrainedTokenizerBase,
+        tokenizer: 'PreTrainedTokenizerBase',
         source_template: str,
         context_length: int,
         context_stride: int,
@@ -439,11 +444,12 @@ class SquadPromptTrainRecipe(BaseRecipe):
         self.chain(pipeline)
 
 
+@Necessary("transformers")
 class SquadPromptValidRecipe(SquadPromptTrainRecipe):
     def __init__(
         self,
         *args,
-        tokenizer: PreTrainedTokenizerBase,
+        tokenizer: 'PreTrainedTokenizerBase',
         target_output_name: Optional[str] = None,
         answer_field: str = "answers",
         **kwargs,
